;; WiFi Car
;;
;; Copyright 2012 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;
;; Control an R/C car with a Wifi module.

.include "msp430x2xx.inc"

; 2.0 ms = 152 interrupts
; 0.3 ms = 23 interrupts
; 0.6 ms = 46 interrupts
; 1.0 ms = 76 interrupts

RAM equ 0x0200

WATCHDOG equ RAM+48

;  r4 =
;  r5 = interupt count
;  r6 = byte coming in from UART
;  r7 = current byte
;  r8 = bit count
;  r9 = interrupt count
; r10 = curr pulse len
; r11 = pointer to servo speeds in interrupt
; r12 =
; r13 =
; r14 =
; r15 = temp in interrupt

  .org 0xc000
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set up stack pointer
  mov.w #0x0400, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

.if 0
  ;; Set MCLK to 16 MHz external crystal
  bic.w #OSCOFF, SR
  bis.b #XTS, &BCSCTL1
  mov.b #LFXT1S_3, &BCSCTL3
  ;mov.b #LFXT1S_3|XCAP_1, &BCSCTL3
test_osc:
  bic.b #OFIFG, &IFG1
  mov.w #0x00ff, r15
dec_again:
  dec r15
  jnz dec_again
  bit.b #(OFIFG), &IFG1
  jnz test_osc
  mov.b #(SELM_3|SELS), &BCSCTL2
.endif

  ;; Set up output pins
  ;; P1.1 = RX
  ;; P1.2 = TX
  ;; P2.0 = Servo 0
  ;; P2.1 = Servo 1
  ;; P2.2 = Horn
  mov.b #0x00, &P1DIR
  mov.b #0x00, &P1OUT
  mov.b #0x06, &P1SEL
  mov.b #0x06, &P1SEL2
  mov.b #0x07, &P2DIR
  mov.b #0x03, &P2OUT
  mov.b #0x00, &P2SEL

  ;; Set up Timer
  mov.w #210, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Setup UART
  mov.b #UCSSEL_2|UCSWRST, &UCA0CTL1
  mov.b #0, &UCA0CTL0
  ;mov.b #0x8a, &UCA0BR0
  ;mov.b #0x00, &UCA0BR1
  ;mov.b #0x14, &UCA0BR0
  ;mov.b #0x01, &UCA0BR1
  mov.b #0xf8, &UCA0BR0
  mov.b #0x05, &UCA0BR1
  bic.b #UCSWRST, &UCA0CTL1

  mov #speed_table, r15
  add #64, r15
  mov.b @r15, &RAM+4  ; Servo 0
  mov.b @r15, &RAM+5  ; Servo 1

  mov.w #50, &WATCHDOG
  mov.w #0, r6
  mov.w #0, r9
  mov.w #0, r10

  ;; Okay, I can be interrupted now
  eint

  ; DEBUG
  ;mov.b #'A', &UCA0TXBUF

main:
  bit.b #UCA0RXIFG, &IFG2
  jz main

  mov.b &UCA0RXBUF, r6
  mov.w #50, &WATCHDOG

.if 0
  mov.b #'*', &UCA0TXBUF
wait_tx:
  bit.b #UCA0TXIFG, &IFG2
  jz wait_tx
.endif

  cmp.b #2, r6
  jnz dont_turn_horn_not_on
  bis.b #4, &P2OUT
  jmp main
dont_turn_horn_not_on:

  cmp.b #1, r6
  jnz dont_turn_horn_not_off
  bic.b #4, &P2OUT
  jmp main
dont_turn_horn_not_off:

  bit.b #0x80, r6
  jz main

  bit.b #0x40, r6
  jnz service_motor

service_steering:
  and.b #0x3f, r6
  rla r6
  add.w #speed_table, r6
  mov.b @r6, RAM+4
  jmp main

service_motor:
  and.b #0x3f, r6
  rla r6
  add.w #speed_table, r6
  mov.b @r6, RAM+5
  jmp main

  ;; Do servo position
  ;mov.b &RAM+2, r7
  ;add.w #speed_table, r7
  ;mov.b @r7, &RAM+4
  ;mov.b @r7, &RAM+5
  ;mov.b @r7, &RAM+6
  ;mov.b @r7, &RAM+7
  ;jmp main

timer_interrupt:
  ;inc.w r5
  inc.w r9
  mov.b &RAM+4, r15
  cmp.w r15, r9
  jl keep_signal_high0
  bis.b #0x01, &P2OUT
keep_signal_high0:
  mov.b &RAM+5, r15
  cmp.w r15, r9
  jl keep_signal_high1
  bis.b #0x02, &P2OUT
keep_signal_high1:

  cmp.w #76*20, r9
  jl not_20_ms_yet
  mov.w #0, r9
  bic.b #0x03, &P2OUT

  dec.w &WATCHDOG
  jnz not_20_ms_yet
  ;; If our watchdog reaches 0, stop motor and center tires.
  mov.w #speed_table, r15
  add.w #64, r15
  mov.b @r15, &RAM+4
  mov.b @r15, &RAM+5
not_20_ms_yet:
  reti 

speed_table:
  db 76, 76, 77, 77, 78, 79, 79, 80
  db 80, 81, 82, 82, 83, 83, 84, 85 
  db 85, 86, 86, 87, 88, 88, 89, 89
  db 90, 91, 91, 92, 92, 93, 94, 94
  db 95, 96, 96, 97, 97, 98, 99, 99
  db 100, 100, 101, 102, 102, 103, 103, 104
  db 105, 105, 106, 106, 107, 108, 108, 109
  db 109, 110, 111, 111, 112, 112, 113, 114
  db 114, 115, 116, 116, 117, 117, 118, 119
  db 119, 120, 120, 121, 122, 122, 123, 123
  db 124, 125, 125, 126, 126, 127, 128, 128
  db 129, 129, 130, 131, 131, 132, 132, 133
  db 134, 134, 135, 136, 136, 137, 137, 138
  db 139, 139, 140, 140, 141, 142, 142, 143
  db 143, 144, 145, 145, 146, 146, 147, 148
  db 148, 149, 149, 150, 151, 151, 152, 153

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



